@@ -264,12 +264,10 @@ g_mime_header_iter_first (GMimeHeaderIter *iter)
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
 	/* make sure we can actually do as requested */
-	if (!iter->hdrlist)
+	if (!iter->hdrlist || list_is_empty (&iter->hdrlist->list))
 		return FALSE;
 	
 	first = (GMimeHeader *) iter->hdrlist->list.head;
-	if (!first->next)
-		return FALSE;
 	
 	iter->version = iter->hdrlist->version;
 	iter->cursor = first;
@@ -294,12 +292,10 @@ g_mime_header_iter_last (GMimeHeaderIter *iter)
 	g_return_val_if_fail (iter != NULL, FALSE);
 	
 	/* make sure we can actually do as requested */
-	if (!iter->hdrlist)
+	if (!iter->hdrlist || list_is_empty (&iter->hdrlist->list))
 		return FALSE;
 	
 	last = (GMimeHeader *) iter->hdrlist->list.tailpred;
-	if (!last->next)
-		return FALSE;
 	
 	iter->version = iter->hdrlist->version;
 	iter->cursor = last;
@@ -598,6 +594,7 @@ g_mime_header_list_clear (GMimeHeaderList *headers)
 	}
 	
 	g_hash_table_remove_all (headers->hash);
+	list_init (&headers->list);
 	
 	g_mime_header_list_set_stream (headers, NULL);
 }
@@ -646,7 +643,7 @@ g_mime_header_list_prepend (GMimeHeaderList *headers, const char *name, const ch
 	g_return_if_fail (name != NULL);
 	
 	header = g_mime_header_new (name, value, -1);
-	list_append (&headers->list, (ListNode *) header);
+	list_prepend (&headers->list, (ListNode *) header);
 	g_hash_table_replace (headers->hash, header->name, header);
 	
 	g_mime_header_list_set_stream (headers, NULL);
