@@ -180,6 +180,8 @@ struct _GMimeParserPrivate {
 	BoundaryStack *bounds;
 };
 
+static const char MBOX_BOUNDARY[6] = "From ";
+#define MBOX_BOUNDARY_LEN 5
 
 static void
 parser_push_boundary (GMimeParser *parser, const char *boundary)
@@ -194,10 +196,10 @@ parser_push_boundary (GMimeParser *parser, const char *boundary)
 	s->parent = priv->bounds;
 	priv->bounds = s;
 	
-	if (!strcmp (boundary, "From ")) {
-		s->boundary = g_strdup ("From ");
-		s->boundarylen = 5;
-		s->boundarylenfinal = 5;
+	if (boundary == MBOX_BOUNDARY) {
+		s->boundary = g_strdup (boundary);
+		s->boundarylen = MBOX_BOUNDARY_LEN;
+		s->boundarylenfinal = MBOX_BOUNDARY_LEN;
 	} else {
 		s->boundary = g_strdup_printf ("--%s--", boundary);
 		s->boundarylen = strlen (boundary) + 2;
@@ -963,6 +965,55 @@ header_parse (GMimeParser *parser, HeaderRaw **tail)
 #endif
 }
 
+enum {
+	SUBJECT = 1 << 0,
+	FROM    = 1 << 1,
+	DATE    = 1 << 2,
+	TO      = 1 << 3,
+	CC      = 1 << 4
+};
+
+static gboolean
+has_message_headers (HeaderRaw *headers)
+{
+	unsigned int found = 0;
+	HeaderRaw *header;
+	
+	header = headers;
+	while (header != NULL) {
+		if (!g_ascii_strcasecmp (header->name, "Subject"))
+			found |= SUBJECT;
+		else if (!g_ascii_strcasecmp (header->name, "From"))
+			found |= FROM;
+		else if (!g_ascii_strcasecmp (header->name, "Date"))
+			found |= DATE;
+		else if (!g_ascii_strcasecmp (header->name, "To"))
+			found |= TO;
+		else if (!g_ascii_strcasecmp (header->name, "Cc"))
+			found |= CC;
+		
+		header = header->next;
+	}
+	
+	return found != 0;
+}
+
+static gboolean
+has_content_headers (HeaderRaw *headers)
+{
+	HeaderRaw *header;
+	
+	header = headers;
+	while (header != NULL) {
+		if (!g_ascii_strcasecmp (header->name, "Content-Type"))
+			return TRUE;
+		
+		header = header->next;
+	}
+	
+	return FALSE;
+}
+
 static int
 parser_step_headers (GMimeParser *parser)
 {
@@ -1044,13 +1095,20 @@ parser_step_headers (GMimeParser *parser)
 						goto next_message;
 					
 					if (priv->headers != NULL) {
-						/* probably the start of the content,
-						 * a broken mailer didn't terminate the
-						 * headers with an empty line. *sigh* */
-						goto content_start;
-					}
-					
-					if (priv->state == GMIME_PARSER_STATE_MESSAGE_HEADERS) {
+						if (priv->state == GMIME_PARSER_STATE_MESSAGE_HEADERS) {
+							if (has_message_headers (priv->headers)) {
+								/* probably the start of the content,
+								 * a broken mailer didn't terminate the
+								 * headers with an empty line. *sigh* */
+								goto content_start;
+							}
+						} else if (has_content_headers (priv->headers)) {
+							/* probably the start of the content,
+							 * a broken mailer didn't terminate the
+							 * headers with an empty line. *sigh* */
+							goto content_start;
+						}
+					} else if (priv->state == GMIME_PARSER_STATE_MESSAGE_HEADERS) {
 						/* Be a little more strict when scanning toplevel message
 						 * headers, but remain lenient with From-lines. */
 						if ((inptr - start) != 4 || strncmp (start, "From ", 5) != 0) {
@@ -1886,7 +1944,7 @@ parser_construct_message (GMimeParser *parser)
 	}
 	
 	if (priv->scan_from) {
-		parser_push_boundary (parser, "From ");
+		parser_push_boundary (parser, MBOX_BOUNDARY);
 		if (priv->respect_content_length && content_length < ULONG_MAX)
 			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
 	}
