@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2011 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -31,6 +31,11 @@
 
 #include "gmime.h"
 
+#ifdef ENABLE_CRYPTOGRAPHY
+#include "gmime-pkcs7-context.h"
+#include "gmime-gpg-context.h"
+#endif
+
 
 /**
  * SECTION: gmime
