@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2011 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -118,7 +118,7 @@ print_mime_part_info (const char *path, GMimeObject *object)
 static void
 print_mime_struct_iter (GMimeMessage *message)
 {
-	const char *jump_to = "4.2.2.2";
+	//const char *jump_to = "4.2.2.2";
 	GMimePartIter *iter;
 	GMimeObject *part;
 	char *path;
